@@ -196,7 +196,7 @@ class Route(object):
         defaultkeys = frozenset([key for key in kargs.keys() \
                                  if key not in reserved_keys])
         for key in defaultkeys:
-            if kargs[key] != None:
+            if kargs[key] is not None:
                 defaults[key] = unicode(kargs[key])
             else:
                 defaults[key] = None
@@ -268,7 +268,16 @@ class Route(object):
                 if not rest:
                     partreg = '(?P<' + var + '>[^%s]+?)' % '/'
                 else:
-                    partreg = '(?P<' + var + '>[^%s]+?)' % ''.join(self.done_chars)
+                    end = ''.join(self.done_chars)
+                    rem = rest
+                    if rem[0] == '\\' and len(rem) > 1:
+                        rem = rem[1]
+                    elif rem.startswith('(\\') and len(rem) > 2:
+                        rem = rem[2]
+                    else:
+                        rem = end
+                    rem = frozenset(rem) | frozenset(['/'])
+                    partreg = '(?P<' + var + '>[^%s]+?)' % ''.join(rem)
             
             if self.reqs.has_key(var):
                 noreqs = False
@@ -596,6 +605,11 @@ class Mapper(object):
         ``decode_errors``
             How to handle errors in the encoding, generally ignoring any chars 
             that don't convert should be sufficient. Defaults to 'ignore'.
+        
+        ``hardcode_names``
+            Whether or not Named Routes result in the default options for the 
+            route being used *or* if they actually force url generation to use
+            the route. Defaults to False.
         """
         self.matchlist = []
         self.maxkeys = {}
@@ -618,6 +632,7 @@ class Mapper(object):
         self.explicit = explicit
         self.encoding = 'utf-8'
         self.decode_errors = 'ignore'
+        self.hardcode_names = False
         if register:
             config = request_config()
             config.mapper = self
@@ -815,7 +830,7 @@ class Mapper(object):
         return None
         
     
-    def generate(self, **kargs):
+    def generate(self, *args, **kargs):
         """Generate a route from a set of keywords
         
         Returns the url text, or None if no URL could be generated.
@@ -844,7 +859,7 @@ class Mapper(object):
         # If the URL didn't depend on the SCRIPT_NAME, we'll cache it
         # keyed by just by kargs; otherwise we need to cache it with
         # both SCRIPT_NAME and kargs:
-        cache_key = unicode(kargs).encode('utf8')
+        cache_key = unicode(args).encode('utf8') + unicode(kargs).encode('utf8')
         if self.environ:
             cache_key_script_name = '%s:%s' % (
                 self.environ.get('SCRIPT_NAME', ''), cache_key)
@@ -868,7 +883,9 @@ class Mapper(object):
         cacheset = False
         cachekey = unicode(keys)
         cachelist = sortcache.get(cachekey)
-        if cachelist:
+        if args:
+            keylist = args
+        elif cachelist:
             keylist = cachelist
         else:
             cacheset = True
@@ -1127,7 +1144,6 @@ class Mapper(object):
         new_path = path + "/new"
         member_path = path + "/:(id)"
         
-        options = {'controller':kwargs.get('controller', controller)}
         options = { 
             'controller': kwargs.get('controller', controller),
             '_member_name': member_name,
